@@ -16,11 +16,6 @@ const console = {
   },
 }
 
-const YAML = {
-  stringify: x => __yaml_stringify__(x),
-  parse: x => JSON.parse(__yaml_parse__(x)),
-}
-
 const skip = Symbol('skip')
 
 function apply(fn, ...args) {
@@ -45,21 +40,16 @@ function sort(x) {
   throw new Error(`Cannot sort ${typeof x}`)
 }
 
+function isFalsely(x) {
+  return x === false || x === null || x === undefined
+}
+
 function filter(fn) {
   return function (x) {
     if (Array.isArray(x)) {
-      return x.filter((v, i) => fn(v, i))
-    } else if (x !== null && typeof x === 'object') {
-      const result = {}
-      for (const [k, v] of Object.entries(x)) {
-        if (fn(v, k)) {
-          result[k] = v
-        }
-      }
-      return result
-    } else {
-      throw new Error(`Cannot filter ${typeof x}`)
+      return x.filter((v, i) => !isFalsely(fn(v, i)))
     }
+    return isFalsely(fn(x))? skip : x
   }
 }
 
@@ -67,15 +57,8 @@ function map(fn) {
   return function (x) {
     if (Array.isArray(x)) {
       return x.map((v, i) => fn(v, i))
-    } else if (x !== null && typeof x === 'object') {
-      const result = {}
-      for (const [k, v] of Object.entries(x)) {
-        result[k] = fn(v, k)
-      }
-      return result
-    } else {
-      throw new Error(`Cannot map over ${typeof x}`)
     }
+    return fn(x)
   }
 }
 
@@ -180,3 +163,8 @@ function toBase64(x) {
 function fromBase64(x) {
   return __fromBase64__(x)
 }
+
+const YAML = {
+  stringify: x => __yaml_stringify__(x),
+  parse: x => JSON.parse(__yaml_parse__(x)),
+}
