@@ -31,11 +31,13 @@ import (
 	"github.com/antonmedv/fx/internal/jsonpath"
 	. "github.com/antonmedv/fx/internal/jsonx"
 	"github.com/antonmedv/fx/internal/theme"
+	"github.com/antonmedv/fx/internal/toml"
 	"github.com/antonmedv/fx/internal/utils"
 )
 
 var (
 	flagYaml     bool
+	flagToml     bool
 	flagRaw      bool
 	flagSlurp    bool
 	flagComp     bool
@@ -43,6 +45,24 @@ var (
 	flagNoInline bool
 )
 
+var flags = []string{
+	"--help",
+	"--raw",
+	"--slurp",
+	"--themes",
+	"--version",
+	"--yaml",
+	"--toml",
+	"--strict",
+	"--no-inline",
+}
+
+func init() {
+	for _, name := range flags {
+		complete.Flags = append(complete.Flags, complete.Reply{name, name, "flag"})
+	}
+}
+
 func main() {
 	if _, ok := os.LookupEnv("FX_PPROF"); ok {
 		f, err := os.Create("cpu.prof")
@@ -89,6 +109,8 @@ func main() {
 			return
 		case "--yaml":
 			flagYaml = true
+		case "--toml":
+			flagToml = true
 		case "--raw", "-r":
 			flagRaw = true
 		case "--slurp", "-s":
@@ -100,13 +122,20 @@ func main() {
 			flagStrict = true
 		case "--no-inline":
 			flagNoInline = true
+		case "--game-of-life":
+			utils.GameOfLife()
+			return
 		default:
 			args = append(args, arg)
 		}
 	}
 
-	if flagYaml && flagRaw {
-		println("Error: can't use both --yaml and --raw flags together")
+	if (flagYaml || flagToml) && flagRaw {
+		println("Error: can't use --yaml/--toml and --raw flags together")
+		os.Exit(1)
+	}
+	if flagYaml && flagToml {
+		println("Error: can't use both --yaml and --toml flags together")
 		os.Exit(1)
 	}
 
@@ -140,7 +169,7 @@ func main() {
 		} else {
 			// $ fx file.json arg*
 			filePath := args[0]
-			src = open(filePath, &flagYaml)
+			src = open(filePath, &flagYaml, &flagToml)
 			engine.FilePath = filePath
 			fileName = filepath.Base(filePath)
 			args = args[1:]
@@ -164,6 +193,18 @@ func main() {
 			return
 		}
 		parser = NewJsonParser(bytes.NewReader(jsonBytes), flagStrict)
+	} else if flagToml {
+		b, err := io.ReadAll(src)
+		if err != nil {
+			panic(err)
+		}
+		jsonBytes, err := toml.ToJSON(b)
+		if err != nil {
+			fmt.Print(err.Error())
+			os.Exit(1)
+			return
+		}
+		parser = NewJsonParser(bytes.NewReader(jsonBytes), flagStrict)
 	} else if flagRaw {
 		parser = NewLineParser(src)
 	} else {
@@ -325,6 +366,7 @@ type model struct {
 	keysIndex             []string
 	keysIndexNodes        []*Node
 	fuzzyMatch            *fuzzy.Match
+	deletePending         bool
 }
 
 type location struct {
@@ -419,6 +461,8 @@ func (m *model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 		}
 
 	case tea.MouseMsg:
+		m.handlePendingDelete(msg)
+
 		switch {
 		case msg.Button == tea.MouseButtonWheelUp:
 			m.up()
@@ -702,7 +746,23 @@ func (m *model) handleShowSelectorKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 	return m, nil
 }
 
+func (m *model) handlePendingDelete(msg tea.Msg) {
+	// Handle potential 'dd' sequence for delete
+	if m.deletePending {
+		if keyMsg, ok := msg.(tea.KeyMsg); ok {
+			if key.Matches(keyMsg, keyMap.Delete) {
+				m.deleteAtCursor()
+				m.deletePending = true
+				return
+			}
+		}
+		m.deletePending = false
+	}
+}
+
 func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
+	m.handlePendingDelete(msg)
+
 	switch {
 	case key.Matches(msg, keyMap.Suspend):
 		m.suspending = true
@@ -1010,6 +1070,8 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 			m.selectNode(loc.node)
 		}
 
+	case key.Matches(msg, keyMap.Delete):
+		m.deletePending = true
 	}
 	return m, nil
 }
@@ -1170,7 +1232,14 @@ func (m *model) prettyPrint(node *Node, isSelected, isRef bool) string {
 		}
 	}
 
-	style := theme.Value(node.Kind, isSelected)
+	var style theme.Color
+
+	if isSelected {
+		style = theme.CurrentTheme.Cursor
+	} else {
+		style = theme.Value(node.Kind)
+	}
+
 	if isRef {
 		style = theme.CurrentTheme.Ref
 	}
@@ -1592,3 +1661,15 @@ func (m *model) open() tea.Cmd {
 		return nil
 	})
 }
+
+// deleteAtCursor deletes the current key/value (node) from the view structure.
+func (m *model) deleteAtCursor() {
+	at, ok := m.cursorPointsTo()
+	if !ok || at == nil {
+		return
+	}
+	if next, ok := DeleteNode(at); ok {
+		m.selectNode(next)
+		m.recordHistory()
+	}
+}
